
class Event(object):
    pass


class UserLoggedIn(Event):
    def __init__(self, request, userid, oauth2_token, userinfo):
        self.request = request
        self.userid = userid
        self.oauth2_token = oauth2_token
        self.userinfo = userinfo
        self.headers = None


class UserLoggedOut(Event):
    def __init__(self, userid):
        self.userid = userid



from collections import namedtuple
import logging

from six.moves.urllib import parse
from pyramid.settings import aslist
from requests.exceptions import RequestException
import requests

from pyramid_google_login import SETTINGS_PREFIX
from pyramid_google_login.exceptions import AuthFailed, ApiError

from zope.interface import Interface

log = logging.getLogger(__name__)

ApiSettings = namedtuple(
    'ApiSettings',
    """
        access_type
        hosted_domain
        id
        landing_route
        landing_url
        scope_list
        secret
        signin_advice
        signin_banner
        user_id_field
    """
    )


class IApiClientFactory(Interface):
    pass


class ApiClient(object):
    """-> https://developers.google.com/accounts/docs/OAuth2WebServer"""
    authorize_endpoint = 'https://accounts.google.com/o/oauth2/auth'
    token_endpoint = 'https://www.googleapis.com/oauth2/v3/token'
    userinfo_endpoint = 'https://www.googleapis.com/oauth2/v2/userinfo'
    domain_users_endpoint = ('https://www.googleapis.com'
                             '/admin/directory/v1/users')

    def __init__(self, request):
        self.request = request

        settings = self.request.registry.settings['googleapi_settings']
        self.id = settings.id
        self.secret = settings.secret
        self.hosted_domain = settings.hosted_domain
        self.access_type = settings.access_type
        self.scope_list = settings.scope_list
        self.user_id_field = settings.user_id_field

    def build_authorize_url(self, state, redirect_uri):
        params = {
            'response_type': 'code',
            'client_id': self.id,
            'redirect_uri': redirect_uri,
            'scope': ' '.join(self.scope_list),
            'state': state,
            'access_type': self.access_type,
        }

        if self.hosted_domain:
            params['hd'] = self.hosted_domain

        authorize_url = '%s?%s' % (
            self.authorize_endpoint,
            parse.urlencode(params)
        )

        return authorize_url

    def exchange_token_from_code(self, redirect_uri):
        if 'error' in self.request.params:
            raise AuthFailed(
                'Error from Google (%s)' % self.request.params['error'])
        try:
            code = self.request.params['code']
        except KeyError as err:
            raise AuthFailed('No authorization code from Google')

        params = {
            'code': code,
            'client_id': self.id,
            'client_secret': self.secret,
            'redirect_uri': redirect_uri,
            'grant_type': 'authorization_code',
        }

        try:
            response = requests.post(self.token_endpoint, data=params)
            response.raise_for_status()
            oauth2_tokens = response.json()

        except RequestException as err:
            raise AuthFailed('Failed to get token from Google (%s)' % err)

        except Exception as err:
            log.warning('Unkown error while calling token endpoint',
                        exc_info=True)
            raise AuthFailed('Failed to get token from Google (unkown error)')

        if 'access_token' not in oauth2_tokens:
            raise AuthFailed('No access_token in response from Google')

        return oauth2_tokens

    def get_userinfo_from_token(self, oauth2_tokens):
        try:
            params = {'access_token': oauth2_tokens['access_token']}
            response = requests.get(self.userinfo_endpoint, params=params)
            response.raise_for_status()
            return response.json()
        except Exception:
            log.warning('Unkown error calling userinfo endpoint',
                        exc_info=True)
            raise AuthFailed('Failed to get userinfo from Google')

    def check_hosted_domain_user(self, userinfo):
        if not self.hosted_domain:
            return

        try:
            user_hosted_domain = userinfo['hd']
        except KeyError:
            raise AuthFailed('Missing hd field from Google userinfo')

        if self.hosted_domain != user_hosted_domain:
            raise AuthFailed('You logged in with an unkown domain '
                             '(%s rather than %s)' % (user_hosted_domain,
                                                      self.hosted_domain))

    def get_user_id_from_userinfo(self, userinfo):
        try:
            user_id = userinfo[self.user_id_field]
        except KeyError:
            raise AuthFailed('Missing user id field from Google userinfo')

        return user_id

    def refresh_access_token(self, refresh_token):
        params = {
            'client_id': self.id,
            'client_secret': self.secret,
            'refresh_token': refresh_token,
            'grant_type': 'refresh_token',
        }

        try:
            response = requests.post(self.token_endpoint, params=params)
            response.raise_for_status()
            oauth2_tokens = response.json()
        except RequestException as err:
            raise AuthFailed(err, 'Failed to get token from Google (%s)' % err)
        except Exception as err:
            log.warning('Unkown error while calling token endpoint',
                        exc_info=True)
            raise AuthFailed(err,
                             'Failed to get token from Google (unknown error)')

        if 'access_token' not in oauth2_tokens:
            raise AuthFailed('No access_token in response from Google')

        return oauth2_tokens

    def get_domain_users(self, access_token, limit=500):
        params = {
            'maxResults': limit,
            'domain': 'ludia.com',
            'viewType': 'domain_public',
            'access_token': access_token
        }
        try:
            response = requests.get(self.domain_users_endpoint, params=params)
            response.raise_for_status()
            return response.json()
        except (ValueError, RequestException) as err:
            raise ApiError(err, 'Failed to get domain users (%s)' % err)


def includeme(config):
    settings = config.registry.settings
    prefix = SETTINGS_PREFIX

    scope_list = set(aslist(settings.get(prefix + 'scopes', '')))
    scope_list.add('email')
    try:
        api_settings = ApiSettings(
            access_type=settings.get(prefix + 'access_type', 'online'),
            hosted_domain=settings.get(prefix + 'hosted_domain'),
            id=settings[prefix + 'client_id'],
            landing_route=settings.get(prefix + 'landing_route'),
            landing_url=settings.get(prefix + 'landing_url'),
            scope_list=scope_list,
            secret=settings[prefix + 'client_secret'],
            signin_advice=settings.get(prefix + 'signin_advice'),
            signin_banner=settings.get(prefix + 'signin_banner'),
            user_id_field=settings.get(prefix + 'user_id_field', 'email'),
            )
    except KeyError as err:
        log.error('Missing configuration setting: %s', err)
        raise

    config.add_settings(googleapi_settings=api_settings)

    config.registry.registerUtility(ApiClient, provided=IApiClientFactory)
    config.add_request_method(new_api_client, 'googleapi', reify=True)


def new_api_client(request):
    return request.registry.getUtility(IApiClientFactory)(request)



import logging

from six.moves.urllib import parse
from pyramid.view import view_config
from pyramid.security import (remember, forget, NO_PERMISSION_REQUIRED)
from pyramid.httpexceptions import HTTPFound

from pyramid_google_login import redirect_to_signin, find_landing_path
from pyramid_google_login.events import UserLoggedIn, UserLoggedOut
from pyramid_google_login.exceptions import AuthFailed

log = logging.getLogger(__name__)


def includeme(config):
    config.add_route('auth_signin', '/auth/signin')
    config.add_route('auth_signin_redirect', '/auth/signin_redirect')
    config.add_route('auth_callback', '/auth/oauth2callback')
    config.add_route('auth_logout', '/auth/logout')

    config.add_static_view('static/pyramid_google_login',
                           'pyramid_google_login:static',
                           cache_max_age=300)

    config.scan(__name__)


def encode_state(params):
    return parse.urlencode(params)


def decode_state(state):
    return parse.parse_qs(state)


@view_config(route_name='auth_signin',
             permission=NO_PERMISSION_REQUIRED,
             renderer='pyramid_google_login:templates/signin.mako')
def signin(request):
    googleapi_settings = request.registry.settings['googleapi_settings']
    message = request.params.get('message')
    url = request.params.get('url')

    if request.authenticated_userid:
        if url:
            return HTTPFound(location=url)
        else:
            return HTTPFound(location=find_landing_path(request))

    if url:
        redirect_url = request.route_url('auth_signin_redirect',
                                         _query={'url': url})
    else:
        redirect_url = request.route_url('auth_signin_redirect')

    return {'signin_redirect_url': redirect_url,
            'message': message,
            'signin_banner': googleapi_settings.signin_banner,
            'signin_advice': googleapi_settings.signin_advice,
            'hosted_domain': googleapi_settings.hosted_domain,
            }


@view_config(route_name='auth_signin_redirect',
             permission=NO_PERMISSION_REQUIRED,)
def signin_redirect(request):
    googleapi = request.googleapi
    redirect_uri = request.route_url('auth_callback')

    state_params = {}
    if 'url' in request.params:
        state_params['url'] = request.params['url']
    state = encode_state(state_params)

    try:
        authorize_url = googleapi.build_authorize_url(state, redirect_uri)
    except AuthFailed as err:
        log.warning('Google Login failed (%s)', err)
        return redirect_to_signin(request, 'Google Login failed (%s)' % err)

    return HTTPFound(location=authorize_url)


@view_config(route_name='auth_callback',
             permission=NO_PERMISSION_REQUIRED)
def callback(request):
    api = request.googleapi
    redirect_uri = request.route_url('auth_callback')
    try:
        oauth2_token = api.exchange_token_from_code(redirect_uri)
        userinfo = api.get_userinfo_from_token(oauth2_token)
        api.check_hosted_domain_user(userinfo)
        userid = api.get_user_id_from_userinfo(userinfo)

    except AuthFailed as err:
        log.warning('Google Login failed (%s)', err)
        return redirect_to_signin(request, 'Google Login failed (%s)' % err)

    except Exception as err:
        log.warning('Google Login failed (%s)', err)
        # Protect against leaking critical information like client_secret
        return redirect_to_signin(request, 'Google Login failed (unkown)')

    # Find the redirect url (fail-safe, the authentication is more important)
    try:
        state_params = decode_state(request.params['state'])
        url = state_params['url'][0]
    except Exception:
        url = find_landing_path(request)

    user_logged_in = UserLoggedIn(request, userid, oauth2_token, userinfo)
    try:
        request.registry.notify(user_logged_in)
    except Exception:
        log.exception('Application crashed processing UserLoggedIn event'
                      '\nuserinfo=%s oauth2_token=%s',
                      userinfo, oauth2_token)
        return redirect_to_signin(request,
                                  'Google Login failed (application error)')

    if user_logged_in.headers:
        headers = user_logged_in.headers
    else:
        headers = remember(request, principal=userid)
    return HTTPFound(location=url, headers=headers)


@view_config(route_name='auth_logout')
def logout(request):
    userid = request.unauthenticated_userid
    if userid is not None:
        event = UserLoggedOut(userid)
        request.registry.notify(event)

    headers = forget(request)
    return redirect_to_signin(request, 'You are logged out!', headers=headers)



import unittest

import mock


class TestIncludeme(unittest.TestCase):

    def test_includeme(self):
        from pyramid_google_login.views import includeme

        config = mock.Mock()

        includeme(config)

        expected_calls = [
            mock.call('auth_signin', '/auth/signin'),
            mock.call('auth_signin_redirect', '/auth/signin_redirect'),
            mock.call('auth_callback', '/auth/oauth2callback'),
            mock.call('auth_logout', '/auth/logout'),
        ]
        config.add_route.assert_has_calls(expected_calls, any_order=True)
        config.add_static_view.assert_called_once_with(
            'static/pyramid_google_login',
            'pyramid_google_login:static',
            cache_max_age=300)
        config.scan.assert_called_once_with('pyramid_google_login.views')


@mock.patch('pyramid_google_login.views.find_landing_path')
class TestSignin(unittest.TestCase):

    def setUp(self):
        self.request = mock.Mock()
        self.request.params = {
        }
        self.settings = {
            'security.google_login.client_id': 'CLIENTID',
            'security.google_login.client_secret': 'CLIENTSECRET',
            'googleapi_settings': mock.Mock(),
        }
        self.request.registry.settings = self.settings

        self.request.authenticated_userid = None
        self.request.route_url.return_value = '/test/url'

    def test_nominal(self, m_find_landing_path):
        from pyramid_google_login.views import signin

        resp = signin(self.request)

        googleapi_settings = (
            self.request.registry.settings['googleapi_settings']
            )

        expected = {'hosted_domain': googleapi_settings.hosted_domain,
                    'message': None,
                    'signin_advice': googleapi_settings.signin_advice,
                    'signin_banner': googleapi_settings.signin_banner,
                    'signin_redirect_url': '/test/url'}

        self.assertEqual(resp, expected)

    def test_authenticated(self, m_find_landing_path):
        from pyramid_google_login.views import signin
        from pyramid.httpexceptions import HTTPFound

        self.request.authenticated_userid = '123'

        resp = signin(self.request)

        self.assertIsInstance(resp, HTTPFound)
        self.assertEqual(resp.location, m_find_landing_path.return_value)

    def test_authenticated_with_url(self, m_find_landing_path):
        from pyramid_google_login.views import signin
        from pyramid.httpexceptions import HTTPFound

        self.request.params['url'] = '/go/there'
        self.request.authenticated_userid = '123'

        resp = signin(self.request)

        self.assertIsInstance(resp, HTTPFound)
        self.assertEqual(resp.location, '/go/there')


@mock.patch('pyramid_google_login.views.redirect_to_signin')
class TestLogout(unittest.TestCase):

    def setUp(self):
        self.request = mock.Mock()
        self.settings = {}
        self.request.registry.settings = self.settings

    def test_nominal(self, m_redirect):
        from pyramid_google_login.views import logout
        from pyramid_google_login.events import UserLoggedOut, Event

        resp = logout(self.request)

        self.assertEqual(resp, m_redirect.return_value)

        self.assertEqual(self.request.registry.notify.called, True)
        event = self.request.registry.notify.call_args[0][0]

        self.assertIsInstance(event, UserLoggedOut)
        self.assertIsInstance(event, Event)
        self.assertEqual(event.userid, self.request.unauthenticated_userid)



import mock

from pyramid_google_login.utility import IApiClientFactory


def includeme(config):
    dummy_api_client = mock.MagicMock()

    config.registry.registerUtility(dummy_api_client,
                                    provided=IApiClientFactory)
    config.add_request_method(new_dummy_api_client, 'googleapi', reify=True)


def new_dummy_api_client(request):
    return request.registry.getUtility(IApiClientFactory)



import mock
from pyramid.config import Configurator
from pyramid.decorator import reify
from requests.exceptions import RequestException

from . import Base


class TestIncludeme(Base):

    def test_includeme_missing_settings(self):
        config = Configurator(settings={})

        with self.assertRaises(KeyError):
            config.include('pyramid_google_login.utility')


class TestUtility(Base):

    @reify
    def googleapi(self):
        self.addCleanup(delattr, self, 'googleapi')
        return self.get_googleapi()

    def get_googleapi(self, path='/'):
        from pyramid_google_login.utility import ApiClient
        return ApiClient(self.get_request(path))


@mock.patch('pyramid_google_login.utility.requests.post')
class TestRefreshAccessToken(TestUtility):

    def test_nominal(self, post):
        response = post.return_value
        response.json.return_value = expected = {
            'access_token': 'i am a token',
        }
        self.assertEqual(
            expected,
            self.googleapi.refresh_access_token('refresh token'),
            )

    def test_raises_request_exception(self, post):
        from pyramid_google_login.exceptions import AuthFailed
        response = post.return_value
        response.raise_for_status.side_effect = RequestException('sh*t!')
        with self.assertRaises(AuthFailed):
            self.googleapi.refresh_access_token('refresh token')

    def test_raises_any_exception(self, post):
        from pyramid_google_login.exceptions import AuthFailed
        response = post.return_value
        response.json.side_effect = ValueError('cr*p!')
        with self.assertRaises(AuthFailed):
            self.googleapi.refresh_access_token('refresh token')

    def test_no_token_in_response(self, post):
        from pyramid_google_login.exceptions import AuthFailed
        response = post.return_value
        response.json.return_value = {
            'not_access_token': 'i am a NOT token',
        }
        with self.assertRaises(AuthFailed):
            self.googleapi.refresh_access_token('refresh token')


@mock.patch('pyramid_google_login.utility.requests.post')
class TestExchangeTokenFromCode(TestUtility):

    def test_nominal(self, post):
        response = post.return_value
        response.json.return_value = expected = {
            'access_token': 'i am a token',
        }

        googleapi = self.get_googleapi('/?code=CODE')
        self.assertEqual(
            expected,
            googleapi.exchange_token_from_code('http://redirect_uri.com')
            )

        endpoint = 'https://www.googleapis.com/oauth2/v3/token'
        params = {
            'code': 'CODE',
            'client_id': 'client id',
            'client_secret': 'client secret',
            'redirect_uri': 'http://redirect_uri.com',
            'grant_type': 'authorization_code',
        }

        post.assert_called_once_with(endpoint, data=params)

    def test_error_in_params(self, post):
        from pyramid_google_login.exceptions import AuthFailed
        googleapi = self.get_googleapi('/?error=f*ck')
        with self.assertRaises(AuthFailed):
            googleapi.exchange_token_from_code('http://redirect_uri.com')

    def test_raises_request_exception(self, post):
        from pyramid_google_login.exceptions import AuthFailed
        response = post.return_value
        response.raise_for_status.side_effect = RequestException('cheis')

        googleapi = self.get_googleapi('/?code=CODE')
        with self.assertRaises(AuthFailed):
            googleapi.exchange_token_from_code('http://redirect_uri.com')

        self.assertTrue(post.called)

    def test_raises_any_exception(self, post):
        from pyramid_google_login.exceptions import AuthFailed
        response = post.return_value
        response.raise_for_status.side_effect = Exception('cheis')

        googleapi = self.get_googleapi('/?code=CODE')
        with self.assertRaises(AuthFailed):
            googleapi.exchange_token_from_code('http://redirect_uri.com')

        self.assertTrue(post.called)

    def test_no_token_in_response(self, post):
        from pyramid_google_login.exceptions import AuthFailed
        response = post.return_value
        response.json.return_value = {
            'not_access_token': 'i am a NOT token',
        }

        googleapi = self.get_googleapi('/?code=CODE')
        with self.assertRaises(AuthFailed):
            googleapi.exchange_token_from_code('http://redirect_uri.com')

        self.assertTrue(post.called)


@mock.patch('pyramid_google_login.utility.requests.get')
class TestUserinfoFromToken(TestUtility):

    def test_nominal(self, get):
        res = self.googleapi.get_userinfo_from_token({'access_token': 'TOKEN'})
        response = get.return_value
        self.assertEqual(response.json.return_value, res)

    def test_raises_exception(self, get):
        from pyramid_google_login.exceptions import AuthFailed
        get.side_effect = Exception('M*RDE!')
        with self.assertRaises(AuthFailed):
            self.googleapi.get_userinfo_from_token({'access_token': 'TOKEN'})


class TestCheckHostedDomainUser(TestUtility):

    def test_nominal(self):
        self.googleapi.check_hosted_domain_user({'hd': 'bob.com'})

    def test_no_hosted_domain(self):
        self.googleapi.hosted_domain = None
        self.assertIsNone(self.googleapi.check_hosted_domain_user({}))

    def test_empty_hosted_domain(self):
        self.googleapi.hosted_domain = ''
        self.assertIsNone(self.googleapi.check_hosted_domain_user({}))

    def test_not_same_domain(self):
        from pyramid_google_login.exceptions import AuthFailed
        with self.assertRaises(AuthFailed):
            self.googleapi.check_hosted_domain_user({'hd': 'not_bob.com'})

    def test_no_hd_in_userinfo(self):
        from pyramid_google_login.exceptions import AuthFailed
        with self.assertRaises(AuthFailed):
            self.googleapi.check_hosted_domain_user({'not_hd': 'whatever'})


class TestGetUserIdFromUserinfo(TestUtility):

    def test_nominal(self):
        self.assertEqual(
            'bob@bob.com',
            self.googleapi.get_user_id_from_userinfo({'email': 'bob@bob.com'})
            )

    def test_key_error(self):
        from pyramid_google_login.exceptions import AuthFailed
        with self.assertRaises(AuthFailed):
            self.googleapi.get_user_id_from_userinfo({'nope': 'whatever'})



from setuptools import setup, find_packages

with open('README.rst') as fd:
    description = fd.read()

VERSION = '1.2.0'  # maintained by release tool

setup(
    name='pyramid_google_login',
    version=VERSION,
    description='Google Login for Pyramid',
    long_description=description,
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Framework :: Pyramid',
        'Topic :: Internet :: WWW/HTTP :: WSGI',
    ],
    keywords=['google', 'login', 'wsgi', 'pyramid', 'auth', 'authentication'],
    author='Pior Bastida',
    author_email='pior@pbastida.net',
    url='https://github.com/ludia/pyramid_google_login',
    license='BSD-derived (http://www.repoze.org/LICENSE.txt)',

    packages=find_packages(),
    package_data={'pyramid_google_login': ['static/*.*', 'templates/*.*']},

    install_requires=['pyramid_mako', 'requests', 'six'],
)



import sys
from . import Base, ApiMockBase


def py3_str(obj):
    if sys.version_info.major > 2:
        return str(obj, 'utf-8')
    return obj


class Test(Base):

    def test_signin(self):
        resp = self.app.get('/auth/signin',
                            status=200)
        expected = '''href="http://localhost/auth/signin_redirect"'''
        self.assertIn(expected, py3_str(resp.body))

    def test_signin_url(self):
        resp = self.app.get('/auth/signin?url=TESTURL',
                            status=200)
        expected = ('''href="http://localhost/auth/signin_redirect'''
                    '''?url=TESTURL"''')
        self.assertIn(expected, py3_str(resp.body))

    def test_signin_message(self):
        resp = self.app.get('/auth/signin?message=TEST+MESSAGE',
                            status=200)
        expected = '''TEST MESSAGE'''
        self.assertIn(expected, py3_str(resp.body))

    def test_signin_redirect(self):
        resp = self.app.get('/auth/signin_redirect?url=TEST%2FURL',
                            status=302)
        location = resp.location

        location_base = 'https://accounts.google.com/o/oauth2/auth?'
        self.assertTrue(location.startswith(location_base))
        self.assertIn('access_type=offline', location)
        self.assertIn('state=url%3DTEST%252FURL', location)
        redir = 'redirect_uri=http%3A%2F%2Flocalhost%2Fauth%2Foauth2callback'
        self.assertIn(redir, location)
        self.assertIn('response_type=code', location)
        self.assertIn('client_id=client+id', location)
        self.assertIn('scope=email', location)
        self.assertIn('hd=bob.com', location)

    def test_callback_error(self):
        resp = self.app.get('/auth/oauth2callback',
                            params={'error': 'ERROR'},
                            status=302)

        expected = ('http://localhost/auth/signin?message='
                    'Google+Login+failed+%28Error+from+Google+%28ERROR%29%29')
        self.assertEqual(resp.location, expected)

    def test_callback_no_code(self):
        resp = self.app.get('/auth/oauth2callback',
                            status=302)

        expected = ('http://localhost/auth/signin?message='
                    'Google+Login+failed+%28No+authorization+code+from'
                    '+Google%29')
        self.assertEqual(resp.location, expected)

    def test_logout(self):
        self.app.set_cookie('auth_tkt', 'whatever')
        resp = self.app.get('/auth/logout',
                            status=302)

        expected = 'http://localhost/auth/signin?message=You+are+logged+out%21'
        self.assertEqual(resp.location, expected)


class TestCallback(ApiMockBase):

    def test_signin_redirect(self):
        from pyramid_google_login.exceptions import AuthFailed

        self.googleapi.build_authorize_url.side_effect = AuthFailed('ooops')

        response = self.app.get('/auth/signin_redirect', status=302)
        self.assertIn(
            'http://localhost/auth/signin?message=Google+Login+failed',
            response.headers.get('Location')
            )

    def test_callback_nominal(self):
        self.googleapi.get_user_id_from_userinfo.return_value = 'bob@bob.com'

        response = self.app.get('/auth/oauth2callback', status=302)
        self.assertEqual(
            'http://localhost/',
            response.headers.get('Location')
            )

    def test_callback_api_raises_exception(self):
        self.googleapi.exchange_token_from_code.side_effect = Exception('wtf')

        response = self.app.get('/auth/oauth2callback', status=302)
        self.assertIn(
            'http://localhost/auth/signin?',
            response.headers.get('Location')
            )
        self.assertFalse(self.googleapi.get_userinfo_from_token.called)

    def test_callback_with_failing_subscriber(self):
        from pyramid_google_login.events import UserLoggedIn

        def subscriber(event):
            raise Exception('WTF')

        self.config.add_subscriber(subscriber, UserLoggedIn)
        self.googleapi.get_user_id_from_userinfo.return_value = 'bob@bob.com'

        response = self.app.get('/auth/oauth2callback', status=302)
        self.assertIn(
            'http://localhost/auth/signin?',
            response.headers.get('Location')
            )



import unittest

from pyramid.config import Configurator
from pyramid.decorator import reify
from pyramid.request import Request
from pyramid.scripting import prepare
from webtest import TestApp

from pyramid_google_login.utility import IApiClientFactory


class Base(unittest.TestCase):

    settings = {
        'security.google_login.client_id': 'client id',
        'security.google_login.client_secret': 'client secret',
        'security.google_login.access_type': 'offline',
        'security.google_login.hosted_domain': 'bob.com',
        }

    @reify
    def config(self):
        self.addCleanup(delattr, self, 'config')
        _config = Configurator(settings=self.settings)
        _config.include('pyramid_google_login')
        return _config

    @reify
    def app(self):
        self.addCleanup(delattr, self, 'app')
        return TestApp(self.config.make_wsgi_app())

    def get_request(self, path='/'):
        self.app  # to bootstrap env
        request = Request.blank(path)
        env = prepare(request=request, registry=self.config.registry)
        self.addCleanup(env['closer'])
        return request


class ApiMockBase(Base):

    @reify
    def config(self):
        _config = super(ApiMockBase, self).config
        _config.commit()
        _config.include('pyramid_google_login.tests')
        return _config

    @reify
    def googleapi(self):
        return self.config.registry.getUtility(IApiClientFactory)
